import java.io.*;
import java.util.*;

class Pair<F, S> {
    public F first;
    public S second;
    public Pair(F f, S s) {
        this.first = f;
        this.second = s;
    }
}

class Test {
    public String s;
    public String descr;

    public Test(String s, String descr) {
        this.s = s;
        this.descr = descr;
    }

    public Test(StringBuilder sb, String descr) {
        this.s = sb.toString();
        this.descr = descr;
    }
}

public class TestGen {
    private static final int MAX_N = 100000;
    private static final int ALPABET = 26;

    int testCount, subtaskCount;
    PrintWriter testsDesc;
    Random random = new Random(0xF00C);

    private String folder;
    private String subtask;
    private int groupNo;
    boolean isSamples = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void setSubtask(String subtask) {
        this.subtask = subtask;
    }

    public void setGroupNo(int groupNo) {
        this.groupNo = groupNo;
    }

    void print(Test test) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);

        System.out.println("Generating #" + testCount + ": " + test.descr);
        out.println(test.s.length());
        out.println(test.s);
        out.close();

        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, 0, ""));
    }

    public void genSamples() throws IOException {
        setFolder("preliminary");
        setSubtask("subtask0");
        isSamples = true;

        System.out.println("==== Generating samples ===");
        print(new Test(".#####.", "sample1"));
        print(new Test(".###.#", "sample2"));
        print(new Test("..###..###", "sample3"));

        testCount = 0;
        isSamples = false;
    }

    void genHandleTests() throws IOException {
    }

    private int[] genAlternate(int n) {
        int c = 2 * (n / 2);
        int[] posLen = new int[c];
        for (int i = 0; i < c; i += 2) {
            posLen[i] = 1;
            posLen[i + 1] = 1;
        }
        return posLen;
    }

    public int[] genSqrt(int n) {
        List<Integer> sqrt = new ArrayList<>();
        int s = n, x = 1;
        while (s >= x) {
            s -= x;
            sqrt.add(x);
            if (s > 0) {
                sqrt.add(1);
                --s;
            }
        }
        Integer[] arrInteger = new Integer[sqrt.size()];
        sqrt.toArray(arrInteger);
        int[] arr = new int[arrInteger.length];
        for (int i = 0; i < arr.length; ++i)
            arr[i] = arrInteger[i];
        return arr;
    }

    public void genSubtask(int n, int groupNo, int points) throws IOException {
        ++subtaskCount;
        String subtask = "subtask" + subtaskCount;
        setSubtask(subtask);
        setFolder("tests");
        setGroupNo(groupNo);
        System.out.println("==== Generating group #" + groupNo + " ===");
        //TODO write here

        int log = (int)(Math.log(n) / Math.log(2));
        int pw = 1<<log;
        print(emptyField(n));
        print(fullField(pw));
        print(fullField(pw - 1));
        print(fullField(pw + 1));

        print(posLenConfig(randInt(n / 2), n / 2));
        print(posLenConfig(randInt(n / 4), n / 4, randInt(n / 4), n / 4));
        print(posLenConfig(randInt(n / 10), n / 10 + 1, randInt(n / 10), n / 10 - 2, randInt(n / 10), n / 10 + 1));
        print(posLenConfig(random.nextInt(n / 2), n / 4));

        print(posLenConfig(genAlternate(n)));
        print(posLenConfig(genAlternate(pw)));
        print(posLenConfig(genAlternate(pw - 1)));
        print(posLenConfig(n / 2 - 1, 1, n / 2 - 1, 1));
        print(posLenConfig(n / 3 - 1, 1, n / 3 - 1, 1, n / 3 - 1, 1));
        print(posLenConfig(divide(n, 5)));
        print(posLenConfig(divide(n / 2, 11)));
        print(posLenConfig(divide(n, 103)));
        print(posLenConfig(divide(n, 500)));

        print(posLenConfig(genSqrt(n)));
        print(posLenConfig(genSqrt(pw)));
        print(posLenConfig(genSqrt(pw - 1)));
        print(posLenConfig(genSqrt(n / 2)));

        for (int i = 0; i < 10; ++i) {
            int nn = randInt(n / 2, n);
            double p = random.nextDouble();
            int numPoints = (int)(nn * p);
            int numDiezs = nn - numPoints;
            int parts = randInt(Math.min(numDiezs, numPoints));
            int[] part2 = genPartition(numPoints, parts);
            int[] part1 = genPartition(numDiezs, parts);
            int[] partsArr = new int[2 * parts];
            for (int j = 0; j < parts; ++j) {
                partsArr[2 * j] = part1[j];
                partsArr[2 * j + 1] = part2[j];
            }
            print(posLenConfig(random.nextBoolean(), partsArr));
        }

        testsDesc.println(String.format("comment_subtask%d=\"n &lt;= %d\"", subtaskCount, n));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
    }

    private int[] divide(int n, int parts) {
        int partLen = n / parts;
        int[] ret = new int[2 * parts];
        for (int i = 0; i < parts; ++i) {
            int x = randInt(partLen / 2, partLen - 1);
            ret[2 * i] = x;
            ret[2 * i + 1] = partLen - x;
        }
        return ret;
    }

    private Test posLenConfig(int... posLen) {
        return posLenConfig(true, posLen);
    }

    private Test posLenConfig(boolean isDiez, int... posLen) {
        StringBuilder sb = new StringBuilder();
        int n = 0;
        for (int i = 0; i < posLen.length; ++i)
            n += posLen[i];

        char c1 = isDiez ? '#' : '.';
        char c2 = isDiez ? '.' : '#';

        for (int i = 0; i < n; ++i)
            sb.append(c2);
        int pos = 0;


        for (int i = 0; i < posLen.length; i += 2) {
            for (int j = pos; j < posLen[i]; ++j)
                sb.setCharAt(j, c1);
            pos += posLen[i];
            if (i + 1 < posLen.length)
                pos += posLen[i + 1];
        }
        return new Test(sb, String.format("pos len config n = %d", n));
    }

    private Test fullField(int n) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < n; ++i)
            sb.append('#');
        return new Test(sb, String.format("full field n = %d", n));
    }

    private Test emptyField(int n) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < n; ++i)
            sb.append('.');
        return new Test(sb, String.format("empty field n = %d", n));
    }

    private void shuffle(List<?> t) {
        Collections.shuffle(t, random);
    }

    private int[] genCombine(int n, int k) {
        if (k > n)
            throw new IllegalArgumentException("k > n");

        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            p.add(i + 1);
        shuffle(p);
        int[] ret = new int[k];
        List<Integer> sublist = p.subList(0, k);
        for (int i = 0; i < k; ++i)
            ret[i] = sublist.get(i);
        Arrays.sort(ret);
        return ret;
    }

    public int[] genPartition(int n, int k) {
        int[] comb = genCombine(n - 1, k - 1);
        int[] ret = new int[k];
        if (k >= 2)
            ret[0] = comb[0];

        for (int i = 1; i + 1 < k; ++i)
            ret[i] = comb[i] - comb[i - 1];
        ret[k - 1] = k >= 2 ?  n - comb[k - 2] : n;
        return ret;
    }

    private String randomString(int len) {
        StringBuilder builder = new StringBuilder();
        for (int i = 0; i < len; ++i)
            builder.append(randomChar());
        return builder.toString();
    }

    private char randomChar() {
        return (char)(random.nextInt(ALPABET) + 'a');
    }

    private int randInt(int n) {
        return random.nextInt(n) + 1;
    }

    private int randInt(int from, int to) {
        return random.nextInt(to - from + 1) + from;
    }

    public void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genSamples();
        genSubtask(1000, 1, 49);
        genSubtask(MAX_N, 2, 51);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}